import 'whatwg-fetch';
import { from, map } from 'rxjs';
import { RequestMapper } from './mappers/request.mapper';
import { checkHttpStatus } from './operators/check-http-status.operator';
import { HttpRequestConfigurations } from './types/http-configurations.enum';
import { HttpInterceptors } from './types/http-interceptors.class';
import { HttpRequest } from './types/http-request.class';
import { HttpResponse } from './types/http-response.class';
export class RxJSHttpClient {
    constructor(reqInterceptors = [], resInterceptors = [], callbackError = () => { }) {
        this._reqInterceptors = new HttpInterceptors(reqInterceptors);
        this._resInterceptors = new HttpInterceptors(resInterceptors);
        this._callbackError = callbackError;
    }
    get(url, config = {}) {
        const request = this._reqInterceptors.execute(new HttpRequest(url, config));
        const configObject = RequestMapper.for(request, HttpRequestConfigurations.GET);
        return from(fetch(url, configObject).catch(this._callbackError)).pipe(map(res => this._resInterceptors.execute(new HttpResponse(res))), checkHttpStatus());
    }
    post(url, config) {
        const request = this._reqInterceptors.execute(new HttpRequest(url, config));
        const configObject = RequestMapper.for(request, HttpRequestConfigurations.POST);
        return from(fetch(url, configObject).catch(this._callbackError)).pipe(map(res => this._resInterceptors.execute(new HttpResponse(res))), checkHttpStatus());
    }
    put(url, config) {
        const request = this._reqInterceptors.execute(new HttpRequest(url, config));
        const configObject = RequestMapper.for(request, HttpRequestConfigurations.PUT);
        return from(fetch(url, configObject).catch(this._callbackError)).pipe(map(res => this._resInterceptors.execute(new HttpResponse(res))), checkHttpStatus());
    }
    patch(url, config) {
        const request = this._reqInterceptors.execute(new HttpRequest(url, config));
        const configObject = RequestMapper.for(request, HttpRequestConfigurations.PATCH);
        return from(fetch(url, configObject).catch(this._callbackError)).pipe(map(res => this._resInterceptors.execute(new HttpResponse(res))), checkHttpStatus());
    }
    delete(url, config) {
        const request = this._reqInterceptors.execute(new HttpRequest(url, config));
        const configObject = RequestMapper.for(request, HttpRequestConfigurations.DELETE);
        return from(fetch(url, configObject).catch(this._callbackError)).pipe(map(res => this._resInterceptors.execute(new HttpResponse(res))), checkHttpStatus());
    }
}
