import { cloneDeep } from 'lodash';
import { urlRegex } from "../helpers/url.regex";
export class HttpRequest {
    constructor(url, config) {
        this.url = new RegExp(urlRegex).test(url) ? url : `${window.location.origin}${url}`;
        this.mode = config.mode || undefined;
        this.cache = config.cache || undefined;
        this.credentials = config.credentials || undefined;
        this.headers = config.headers || {};
        this.redirect = config.redirect || undefined;
        this.referrer = config.referrer || undefined;
        this.body = config.body || undefined;
    }
    get headers() {
        return this._headers;
    }
    set headers(headers) {
        if (headers.hasOwnProperty('Content-Type')) {
            this._headers = headers;
        }
        else {
            this._headers = Object.assign({ 'Content-Type': 'application/json' }, headers);
        }
    }
    clone() {
        return cloneDeep(this);
    }
}
